#version 330

layout (triangles) in;							// Format of the input primitive
layout (triangle_strip, max_vertices = 3) out;	// Format of the output primitive

uniform vec3 viewpoint;

out vec4 color;

//----------------------------------------------------------------
// TODO:
// 1) Compute face normal as cross product of triangle edge vectors.
// 2) Compute vector from the face (v0) to the viewpoint.
// 3) Decide whether the polygon is front facing (angle between normal and view vector is less than 90).
// 4) Emit primitive only in that case.
//----------------------------------------------------------------

void main()
{
	// Calculate two vectors in the plane of the input triangle
	vec3 ab = gl_in[1].gl_Position.xyz - gl_in[0].gl_Position.xyz;
	vec3 ac = gl_in[2].gl_Position.xyz - gl_in[0].gl_Position.xyz;

	// Calculate the face normal and the view direction vector
	vec3 normal = normalize(cross(ac, ab));
	vec3 vt = normalize(viewpoint - gl_in[0].gl_Position.xyz);

	// Take the dot product of the normal with the view direction
	float d = dot(vt, normal);

	// Emit a primitive only if the sign of the dot product is positive
	if(d > 0.f)
	{
		for(int i = 0; i < gl_in.length(); i++)
		{
			gl_Position = gl_in[i].gl_Position;
			color = vec4(0.f, 0.f, 0.f, 1.f);
			color[i] = 1.f;
			EmitVertex();
		}
 
		EndPrimitive();
	}
}