#version 330

layout (triangles) in;						// Format of the input primitive
layout (line_strip, max_vertices = 8) out;	// Format of the output primitive

layout (std140) uniform Matrices
{
	mat4 m_pvm;
	mat4 m_viewModel;
	mat3 m_normal;
};

uniform vec4 vertexNormalColor = vec4(0.8f, 0.1f, 0.1f, 1.f);	// Color of vertex normal
uniform float vertexNormalScale = 0.05f;						// Size of the vertex normal
uniform vec4 faceNormalColor = vec4(0.1f, 0.8f, 0.1f, 1.f);		// Color of face normal
uniform float faceNormalScale = -0.1f;							// Size of face normal

in Data
{
	vec3 normal;
} vertexData[];

out vec4 color;

//----------------------------------------------------------------
// TODO:
// 1) Emit lines for vertex normals: line from original vertex to vertex displaced along the normal.
// 2) Caculate face normal and triangle centroid (from previous examples).
// 3) Emit face normal lines: line from centroid and displaced centroid.
// 4) Explain how perspective clipping affects the result. Compare with tessallate.
// 5) Why is faceNormalScale negative?
//----------------------------------------------------------------

void main()
{
	// Lines for the vertices normals. Two output vertices per input vertex.
	for(int i = 0; i < gl_in.length(); i++)
	{
		color = vertexNormalColor;
		gl_Position = m_pvm * gl_in[i].gl_Position;
		EmitVertex();

		color = vec4(0.f);
		gl_Position = m_pvm * (gl_in[i].gl_Position + vec4(normalize(vertexData[i].normal) * vertexNormalScale, 0.f));
		EmitVertex();

		EndPrimitive();
	}

	// Triangle face normal (from triangle's centroid)
	// Do the perspective clipping now, so that line length is not affected by it
	vec4 cent = (gl_in[0].gl_Position + gl_in[1].gl_Position + gl_in[2].gl_Position) / 3.f;
	
	// Calculate two vectors in the plane of the input triangle
	vec3 ab = gl_in[1].gl_Position.xyz - gl_in[0].gl_Position.xyz;
	vec3 ac = gl_in[2].gl_Position.xyz - gl_in[0].gl_Position.xyz;

	// Calculate the face normal
	vec3 normal = normalize(cross(ac, ab));

	// Line for the face normal
	gl_Position = m_pvm * cent;
	color = faceNormalColor;
	EmitVertex();

	gl_Position = m_pvm * (cent + vec4(normal * faceNormalScale, 0.f));
	color = vec4(0.f);
	EmitVertex();

	//EndPrimitive();
}